# -*- coding: utf-8 -*-

from __future__ import absolute_import

from benedict.serializers.abstract import AbstractSerializer
from benedict.utils import type_util

try:
    from configparser import ConfigParser
    from configparser import DEFAULTSECT as default_section
except ImportError:
    from ConfigParser import ConfigParser
    default_section = 'DEFAULT'

from six import PY2, StringIO


class INISerializer(AbstractSerializer):

    def __init__(self):
        super(INISerializer, self).__init__()

    @staticmethod
    def _get_section_option_value(parser, section, option):
        value = None
        funcs = [
            parser.getint,
            parser.getfloat,
            parser.getboolean,
            parser.get,
        ]
        for func in funcs:
            try:
                value = func(section, option)
                break
            except ValueError:
                continue
        return value

    def decode(self, s, **kwargs):
        parser = ConfigParser(**kwargs)
        if PY2:
            parser.readfp(StringIO(s))
        else:
            parser.read_string(s)
        data = {}
        for option, _ in parser.defaults().items():
            data[option] = self._get_section_option_value(
                parser, default_section, option)
        for section in parser.sections():
            data[section] = {}
            for option, _ in parser.items(section):
                data[section][option] = self._get_section_option_value(
                    parser, section, option)
        return data

    def encode(self, d, **kwargs):
        parser = ConfigParser(**kwargs)
        for key, value in d.items():
            if not type_util.is_dict(value):
                parser.set(default_section, key, '{}'.format(value))
                continue
            section = key
            parser.add_section(section)
            for option_key, option_value in value.items():
                parser.set(section, option_key, '{}'.format(option_value))
        str_data = StringIO()
        parser.write(str_data)
        return str_data.getvalue()



# -*- coding: utf-8 -*-


class BaseDict(dict):

    _dict = None
    _pointer = False

    def __init__(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], dict) and args[0]:
            self._dict = args[0].dict() if issubclass(
                type(args[0]), BaseDict) else args[0]
            self._pointer = True
            super(BaseDict, self).__init__(self._dict)
            return
        self._dict = None
        self._pointer = False
        super(BaseDict, self).__init__(*args, **kwargs)

    def __bool__(self):
        if self._pointer:
            return bool(self._dict)
        return len(self.keys()) > 0

    def __contains__(self, key):
        if self._pointer:
            return key in self._dict
        return super(BaseDict, self).__contains__(key)

    def __delitem__(self, key):
        if self._pointer:
            del self._dict[key]
            return
        super(BaseDict, self).__delitem__(key)

    def __eq__(self, other):
        if self._pointer:
            return self._dict == other
        return super(BaseDict, self).__eq__(other)

    def __getitem__(self, key):
        if self._pointer:
            return self._dict[key]
        return super(BaseDict, self).__getitem__(key)

    def __iter__(self):
        if self._pointer:
            return iter(self._dict)
        return super(BaseDict, self).__iter__()

    def __len__(self):
        if self._pointer:
            return len(self._dict)
        return super(BaseDict, self).__len__()

    def __nonzero__(self):
        # python 2
        return self.__bool__()

    def __repr__(self):
        if self._pointer:
            return repr(self._dict)
        return super(BaseDict, self).__repr__()

    def __setitem__(self, key, value):
        if self._pointer:
            self._dict[key] = value
            return
        super(BaseDict, self).__setitem__(key, value)

    def __str__(self):
        if self._pointer:
            return str(self._dict)
        return super(BaseDict, self).__str__()

    def __unicode__(self):
        if self._pointer:
            return unicode(self._dict)
        return '{}'.format(self)

    def clear(self):
        if self._pointer:
            self._dict.clear()
            return
        super(BaseDict, self).clear()

    def copy(self):
        if self._pointer:
            return self._dict.copy()
        return super(BaseDict, self).copy()

    def dict(self):
        if self._pointer:
            return self._dict
        return self

    def get(self, key, default=None):
        if self._pointer:
            return self._dict.get(key, default)
        return super(BaseDict, self).get(key, default)

    def items(self):
        if self._pointer:
            return self._dict.items()
        return super(BaseDict, self).items()

    def keys(self):
        if self._pointer:
            return self._dict.keys()
        return super(BaseDict, self).keys()

    def pop(self, key, *args):
        if self._pointer:
            return self._dict.pop(key, *args)
        return super(BaseDict, self).pop(key, *args)

    def setdefault(self, key, default=None):
        if self._pointer:
            return self._dict.setdefault(key, default)
        return super(BaseDict, self).setdefault(key, default)

    def update(self, other):
        if self._pointer:
            self._dict.update(other)
            return
        super(BaseDict, self).update(other)

    def values(self):
        if self._pointer:
            return self._dict.values()
        return super(BaseDict, self).values()



# -*- coding: utf-8 -*-

from benedict.serializers import (
    get_format_by_path, get_serializer_by_format, get_serializers_extensions, )

import fsutil


def autodetect_format(s):
    if is_url(s) or is_filepath(s):
        return get_format_by_path(s)
    return None


def decode(s, format, **kwargs):
    serializer = get_serializer_by_format(format)
    if not serializer:
        raise ValueError('Invalid format: {}.'.format(format))
    decode_opts = kwargs.copy()
    data = serializer.decode(s.strip(), **decode_opts)
    return data


def encode(d, format, **kwargs):
    serializer = get_serializer_by_format(format)
    if not serializer:
        raise ValueError('Invalid format: {}.'.format(format))
    s = serializer.encode(d, **kwargs)
    return s


def is_data(s):
    return (len(s.splitlines()) > 1)


def is_filepath(s):
    if any([s.endswith(ext) for ext in get_serializers_extensions()]):
        return True
    return fsutil.is_file(s)


def is_url(s):
    return any([s.startswith(protocol)
                for protocol in ['http://', 'https://']])


def read_content(s):
    # s -> filepath or url or data
    if is_data(s):
        # data
        return s
    elif is_url(s):
        # url
        return read_url(s)
    elif is_filepath(s):
        # filepath
        return read_file(s)
    # one-line data?!
    return s


def read_file(filepath, **options):
    if fsutil.is_file(filepath):
        return fsutil.read_file(filepath, **options)
    return None


def read_url(url, **options):
    return fsutil.read_file_from_url(url, **options)


def write_file(filepath, content, **options):
    fsutil.write_file(filepath, content, **options)
